package Concordance;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.StringTokenizer;
/**
 * @author Jonathan Field jof2@aber.ac.uk 073717922
 * @version 1.00
 * 
 * The FileInputReader Class provides functionality to allow two text files to be processed.
 * First an index file containing the words to be looked up is loaded in, this is followed by a reference text file.
 * The results are stored in a HashTable to allow fast searching, the line numbers and context are also added to a LinkedList to provide more functionality.
 */


public class FileInputReader {

	private int lineNumber;
	private static final int LINE_DEFAULT = 1;
	private Hashtable<String, LinkedList<ConcordanceResult>> indexTable = new Hashtable<String, LinkedList<ConcordanceResult>>();
	//private LinkedList<String> list = new LinkedList<String>();
	private Conversion convert;

	public FileInputReader() {

		this.lineNumber = LINE_DEFAULT;
		this.convert = new Conversion();

	}

	/**
	 * This method creates an input stream to allow the index text file to be read in. Each line is read in sequence,
	 * the index word is converted to lower case to allow for easier reference to the text later on. A default value is also given to the LinkedList as 
	 * the lineNumbers are as of yet no appropriate
	 * 
	 * The input stream is then closed and the next the next method is called to process the text file and compare it to the index words in the hashtable
	 * 
	 * @param indexToLoad
	 * 	A String reference of a filepath to the desired index file to load in .txt format
	 * @param textToLoad
	 * 	A String reference of a filepath to the desired text file to load in .txt format
	 */
	public void processInputIndex(String indexToLoad, String textToLoad) {
		
		try {
			FileReader fr = new FileReader(indexToLoad);

			BufferedReader in = new BufferedReader(fr);

			String str;

			while ((str = in.readLine()) != null) {

				LinkedList<ConcordanceResult> indexList = new LinkedList<ConcordanceResult>();

				this.indexTable.put(str.toLowerCase(), indexList);

			}
			
			in.close();

		} catch (Exception e) {

			System.out.println(e.getMessage());

		}

		try {
			
			
			this.processInputText(textToLoad);
		
		} catch (IOException e) {

			e.printStackTrace();
			
		}
		
		

	}

	
	/**
	 * Creates an input stream to read in a reference text file, this file is then tokenized (split up into individual words), this allows each word to be compared to the index words contained in the hashtable.
	 * If there is a match between a word being processed and a word in the hashtable then the line number and context are added to the hashtable.
	 * 
	 * @param textToLoad
	 * 	A String reference of a filepath to the desired text file to load in .txt format
	 * @throws IOException
	 * 	Exception thrown if file is not found
	 */
	public void processInputText(String textToLoad) throws IOException {

		FileReader fr = new FileReader(textToLoad);

		BufferedReader in = new BufferedReader(fr);

		String str;

		while ((str = in.readLine()) != null) {

			StringTokenizer st = new StringTokenizer(str);

			while (st.hasMoreTokens()) {

				String temp = this.convert.removePuncuation(st.nextToken()
						.toLowerCase());

				if (this.indexTable.containsKey(temp)) {

					LinkedList<ConcordanceResult> lineAndContextList = this.indexTable.get(temp);

					ConcordanceResult result = new ConcordanceResult(str,
							this.lineNumber);

					lineAndContextList.add(result);
					this.indexTable.put(temp, lineAndContextList);

				}

			}
			this.lineNumber++;

		}

		in.close();

	}

	/**
	 * Returns the HashTable 
	 * 
	 * @return 
	 * Returns the index HashTable
	 */
	public Hashtable<String, LinkedList<ConcordanceResult>> returnHashTable() {

		return this.indexTable;

	}

	
	/**
	 * This method provides a sort, the items are added to an ArrayList and then Collections.sort() is called to sort them into alphabetical order,
	 * the sorted list is then returned to be displayed on the GUI.
	 * @return
	 * Returns sorted list of index words
	 */
	public ArrayList<String> returnList() {

		ArrayList<String> tempList = new ArrayList<String>();

		for (String key : this.indexTable.keySet()) {
			tempList.add(key);
		}

		Collections.sort(tempList, String.CASE_INSENSITIVE_ORDER);

		return tempList;

	}
	
	
	/**
	 * This method returns a LinkedList of ConcordanceResult, these contain the context and line numbers needed to provide extra information about the index word
	 * @param selectedIndex
	 * The selected index of a JList to show the users desired choice of index word
	 * @return
	 */
	public LinkedList<ConcordanceResult> getResultsList(String selectedIndex) {

		LinkedList<ConcordanceResult> resultsList = this.indexTable.get(selectedIndex);

		return resultsList;

	}

	public int getLineNumber() {
		return lineNumber;
	}

	public void setLineNumber(int lineNumber) {
		this.lineNumber = lineNumber;
	}
	
	

}
